@@ -34,28 +34,34 @@
 /*
  * Class:     org_liblouis_liblouisutdml
  * Method:    initialize
- * Signature: (Ljava/lang/String;Ljava/lang/String;)V
+ * Signature: (Ljava/lang/String;Ljava/lang/String;Ljava/lang/String;)V
  */
 JNIEXPORT void JNICALL Java_org_liblouis_liblouisutdml_initialize
   (JNIEnv *env, jclass class, jstring dataPath, jstring
-writeablePath)
+writeablePath, jstring logfile)
 {
   const jbyte *dataPathX = NULL;
   const jbyte *writeablePathX = NULL;
+  const jbyte *logfileX = NULL;
   dataPathX = (*env)->GetStringUTFChars (env, dataPath, NULL);
   if (dataPathX == NULL)
     goto release;
   writeablePathX = (*env)->GetStringUTFChars (env, writeablePath, NULL);
   if (writeablePathX == NULL)
     goto release;
+  logfileX = (*env)->GetStringUTFChars (env, logfile, NULL);
+  if (logfileX == NULL)
+    goto release;
   lou_setDataPath (dataPathX);
   lbu_setWriteablePath (writeablePathX);
-  read_configuration_file (NULL, NULL, NULL, 0);
+  read_configuration_file (NULL, logfileX, NULL, 0);
 release:
   if (dataPathX != NULL)
     (*env)->ReleaseStringUTFChars (env, dataPath, dataPathX);
   if (writeablePathX != NULL)
     (*env)->ReleaseStringUTFChars (env, writeablePath, writeablePathX);
+  if (logfileX != NULL)
+    (*env)->ReleaseStringUTFChars (env, logfile, logfileX);
 }
 
 /*
@@ -693,7 +699,7 @@ JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_file2brl
   strcpy (configFileList, "preferences.cfg");
   strcpy (inputFileName, "stdin");
   strcpy (outputFileName, "stdout");
-  strcpy (logFileName, "file2brl.log");
+  logFileName[0] = 0;
   if (numArgs != 0)
     {
       getArg (env, obj, args, -1);
@@ -708,7 +714,8 @@ JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_file2brl
 	      switch (curArg[1])
 		{
 		case 'l':
-		  strcpy (logFileName, getArg (env, obj, args, k + 1));
+		  strcpy (logFileName, lbu_getWriteablePath ());
+		  strcat (logFileName, getArg (env, obj, args, k + 1));
 		  k += 2;
 		  break;
 		case 't':
@@ -742,7 +749,7 @@ JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_file2brl
 		  k++;
 		  break;
 		default:
-		  lou_logPrint ("invalid argument%s", curArg);
+		  lou_logPrint ("invalid argument '%s'", curArg);
 		  return JNI_FALSE;
 		}
 	      continue;
@@ -762,7 +769,7 @@ JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_file2brl
 		}
 	      else
 		{
-		  lou_logPrint ("extra operand: %s\n",
+		  lou_logPrint ("extra operand: '%s'\n",
 				getArg (env, obj, args, k + 2));
 		  return JNI_FALSE;
 		}
@@ -771,7 +778,8 @@ JNIEXPORT jboolean JNICALL Java_org_liblouis_liblouisutdml_file2brl
 	}
       getArg (env, obj, args, -1);
     }
-  lou_logFile (logFileName);
+  if (logFileName[0] != 0)
+    lou_logFile (logFileName);
   if (whichProc == 0)
     whichProc = 'x';
   if (configSettings != NULL)
@@ -788,7 +796,7 @@ configSettings, 0))
     {
       if (!(inputFile = fopen (inputFileName, "rb")))
 	{
-	  lou_logPrint ("Can't open file %s.\n", inputFileName);
+	  lou_logPrint ("Can't open input file '%s'.\n", inputFileName);
 	  lou_logEnd ();
 	  return JNI_FALSE;
 	}
@@ -798,7 +806,7 @@ configSettings, 0))
   /*Create somewhat edited temporary file to facilitate use of stdin. */
   strcpy (tempFileName, lbu_getWriteablePath ());
   strcat (tempFileName, "file2brl.temp");
-  if (!(tempFile = fopen (tempFileName, "w")))
+  if (!(tempFile = fopen (tempFileName, "wb")))
     {
       lou_logPrint ("Can't open temporary file.\n");
       lou_logEnd ();
@@ -860,7 +868,7 @@ configSettings, 0))
 	if (charsRead == 0)
 	  {
 	    if (ch != '<' && whichProc == 'x')
-	      whichProc = 't';
+	      whichProc = 'T';
 	    nch = fgetc (inputFile);
 	    if (!(mode & htmlDoc) && whichProc == 'x' && nch != '?')
 	      fprintf (tempFile, "%s\n", ud->xml_header);
