@@ -40,11 +40,17 @@ static int doConfigfile (xmlNode * node);
 int
 examine_document (xmlNode * node)
 {
-/*Examine the DOM tree, add semantic attributes and set indicators.*/
+/*Examine the parse tree, add semantic attributes and set indicators.*/
   xmlNode *child;
   if (node == NULL)
     return 0;
   ud->stack[++ud->top] = set_sem_attr (node);
+  if (ud->format_for == utd)
+  {
+  unsigned char *name = get_sem_name (node);
+  if (name[0] != 0)
+  xmlNewProp (node, (xmlChar *)"semantics", (xmlChar *)name);
+  }
   switch (ud->stack[ud->top])
     {
     case skip:
