@@ -120,7 +120,8 @@ processXmlDocument (const char *inputDoc, int length)
 	  if (ud->doc == NULL)
 	    {
 	      lou_logPrint
-		("Document could not be processed, probably  malformed");
+		("Document could not be processed; may be \
+malformed or contain illegal characters");
 	      cleanupLibxml ();
 	      return 0;
 	    }
@@ -517,6 +518,7 @@ lbu_free ()
 {
 /* Free all memory used by liblouisutdml. You MUST call this function at 
 * the END of your application.*/
+  lou_logEnd ();
   lou_free ();
   destroy_semantic_table ();
   if (ud != NULL)
