@@ -134,19 +134,29 @@ file_exists (const char *completePath)
 }
 
 int
-find_file (const char *fileList, char *filePath)
+find_file (const char *fileName, char *filePath)
 {
   struct stat statInfo;
   char trialPath[MAXNAMELEN];
-  int commaPos;			/*for file lists */
   int listLength;
   int k;
   int currentListPos = 0;
+  int nameLength = strlen (fileName);
   int pathLength;
   filePath[0] = 0;
-  for (commaPos = 0; fileList[commaPos] && fileList[commaPos] != ',';
-       commaPos++);
-/*Process path list*/
+  for (k = 0; k < nameLength && fileName[k] != ud->file_separator; k++);
+  if (k != nameLength)
+    {
+      /* This name contains a path */
+      if (stat (fileName, &statInfo) != -1)
+	{
+	  strcpy (filePath, fileName);
+	  return 1;
+	}
+      else
+	return 0;
+    }
+  /*Process path list */
   listLength = strlen (ud->path_list);
   for (k = 0; k < listLength; k++)
     if (ud->path_list[k] == ',')
@@ -157,10 +167,10 @@ find_file (const char *fileList, char *filePath)
       if (trialPath[strlen (trialPath) - 1] != ud->file_separator)
 	strcat (trialPath, pathEnd);
       pathLength = strlen (trialPath);
-      strncat (trialPath, (char *) fileList, commaPos);
+      strcat (trialPath,  fileName);
       if (stat (trialPath, &statInfo) != -1)
 	{
-	  strcpy (&trialPath[pathLength], fileList);
+	  strcpy (&trialPath[pathLength], fileName);
 	  strcpy (filePath, trialPath);
 	  return 1;
 	}
@@ -172,10 +182,10 @@ find_file (const char *fileList, char *filePath)
       if (trialPath[strlen (trialPath) - 1] != ud->file_separator)
 	strcat (trialPath, pathEnd);
       pathLength = strlen (trialPath);
-      strncat (trialPath, (char *) fileList, commaPos);
+      strcat (trialPath,  fileName);
       if (stat (trialPath, &statInfo) != -1)
 	{
-	  strcpy (&trialPath[pathLength], fileList);
+	  strcpy (&trialPath[pathLength], fileName);
 	  strcpy (filePath, trialPath);
 	  return 1;
 	}
@@ -191,10 +201,10 @@ find_file (const char *fileList, char *filePath)
 	  if (trialPath[strlen (trialPath) - 1] != ud->file_separator)
 	    strcat (trialPath, pathEnd);
 	  pathLength = strlen (trialPath);
-	  strncat (trialPath, (char *) fileList, commaPos);
+	  strcat (trialPath,  fileName);
 	  if (stat (trialPath, &statInfo) != -1)
 	    {
-	      strcpy (&trialPath[pathLength], fileList);
+	      strcpy (&trialPath[pathLength], fileName);
 	      strcpy (filePath, trialPath);
 	      return 1;
 	    }
@@ -1097,6 +1107,8 @@ compileConfig (FileInfo * nested)
 	      "17",
 	      "runningHead",
 	      "18",
+	      "emphasis",
+	      "19",
 	      NULL
 	    };
 	    static const char *formats[] = {
@@ -1216,6 +1228,17 @@ compileConfig (FileInfo * nested)
 		    if ((k = checkValues (nested, yesNo)) != NOTFOUND)
 		      style->runningHead = k;
 		    break;
+		  case 19:
+		  {
+		  sem_act action = find_semantic_number (nested->value);
+		  if (action == italicx || action == boldx || action == 
+		  underlinex || action == compbrl)
+		  style->emphasis = action;
+		  else
+		  configureError (nested, "invalid emphasis '%s'", 
+		  nested->value);
+		  }
+		  break;
 		  default:
 		    configureError (nested, "Program error in readconfig.c");
 		    continue;
@@ -1422,7 +1445,8 @@ read_configuration_file (const char *configFileList, const char
 	  return 0;
 	}
       ud->cells_per_line = (ud->page_right - ud->page_left) / ud->cell_width;
-      ud->lines_on_page = (ud->page_bottom - ud->page_top) / ud->normal_line;
+      ud->lines_per_page = (ud->page_bottom - ud->page_top) / 
+      ud->normal_line;
       ud->back_text = textDevice;
       ud->back_line_length = 70;
     }
