@@ -40,6 +40,8 @@ static const char *semNames[] = {
   "uncontracted",
   "contracted",
   "pagenum",
+  "genpurp1",
+  "genpurp2",
   "pagebreak",
   "attrtotext",
   "runninghead",
@@ -98,7 +100,8 @@ static const char *semNames[] = {
   "utdnewpage",
   "utdnewline",
   "utdbrlonly",
-  "utdgraphic",
+  "utddispimg",
+  "utdinlnimg",
   "end_all",
 NULL
 };
