@@ -1247,14 +1247,14 @@ get_sem_name (xmlNode * node)
       strncat (key, macro, k);
     }
   else if (nodeEntry->style != NULL)
-  {
-  strcpy (key, "style,");
-    strcat (key, nodeEntry->style->name);
+    {
+      strcpy (key, "style,");
+      strcat (key, nodeEntry->style->name);
     }
   else
     {
-    strcpy (key, "action,");
-    strcat (key, semNames[nodeEntry->semNum]);
+      strcpy (key, "action,");
+      strcat (key, semNames[nodeEntry->semNum]);
     }
   return key;
 }
@@ -1295,36 +1295,51 @@ get_attr_value (xmlNode * node)
   return xmlGetProp (node, attrName);
 }
 
+void
+shortenStack ()
+{
+  int k;
+  int kk = 0;
+  for (k = ud->top / 2; k <= ud->top; k++)
+    ud->stack[kk++] = ud->stack[k];
+  ud->top = kk - 1;
+}
+
 sem_act
 push_sem_stack (xmlNode * node)
 {
-  if (ud->top > (STACKSIZE - 2) || ud->top < -1)
-    {
-      ud->top = 1;
-      ud->stack[ud->top] = no;
-      return no;
-    }
+  if (ud->top < -1)
+    ud->top = -1;
+  if (ud->top > (STACKSIZE - 3))
+    shortenStack ();
   return (ud->stack[++ud->top] = get_sem_attr (node));
 }
 
+sem_act
+push_action (sem_act action)
+{
+  if (ud->top < -1)
+    ud->top = -1;
+  if (ud->top > (STACKSIZE - 3))
+    shortenStack ();
+  return (ud->stack[++ud->top] = action);
+}
+
 sem_act
 pop_sem_stack ()
 {
   if (ud->top < 0)
     {
-      ud->top = 1;
-      ud->stack[ud->top] = no;
+      ud->top = -1;
+      ud->stack[++ud->top] = no;
       return no;
     }
   ud->top--;
   if (ud->top > -1)
     return ud->stack[ud->top];
-  ud->top = 1;
-  ud->stack[ud->top] = no;
   return no;
 }
 
-
 static void
 addNewEntries (const xmlChar * newEntry)
 {
@@ -1651,6 +1666,18 @@ doSemanticActions ()
     case pagenum:
       do_pagenum ();
       break;
+    case italicx:
+      push_action (italicx);
+      break;
+    case boldx:
+      push_action (boldx);
+      break;
+    case underlinex:
+      push_action (underlinex);
+      break;
+    case compbrl:
+      push_action (compbrl);
+      break;
     default:
       macroError ("semantic action %s is not supported in a macro",
 		  semNames[semNum]);
@@ -1665,7 +1692,7 @@ compileMacro ()
   xmlChar compiledMacro[4 * MAXNAMELEN];
   int unPos = 0;
   int pos = 0;
-  memset (compiledMacro, sizeof (compiledMacro), 0);
+  memset (compiledMacro, 0, sizeof (compiledMacro));
   while (unPos < macroLength)
     {
       if (isalpha (macro[unPos]))
