@@ -33,6 +33,14 @@
 #include <string.h>
 #include "louisutdml.h"
 
+extern void widecharcpy (widechar * to, const widechar * from, int length);
+extern void widestrcpy (widechar * to, const widechar * from);
+extern void unsignedcharcpy (char * to, const char * from, int length);
+extern void save_translated_buffer (void);
+extern void restore_translated_buffer (void);
+extern void contents_save_state (void);
+extern void contents_restore_state (void);
+
 static int dont_split = 0;
 static int dont_split_status = 0;
 static int keep_with_next = 0;
@@ -202,7 +210,7 @@ transcribe_paragraph (xmlNode * node, int action)
     case skip:
       if (action != 0)
 	pop_sem_stack ();
-      return 0;
+      return 1;
     case markhead:
       ud->head_node = node;
       pop_sem_stack ();
@@ -215,9 +223,9 @@ transcribe_paragraph (xmlNode * node, int action)
 	  lou_logPrint ("Cannot open main table %s", ud->main_braille_table);
 	  return 0;
 	}
-      if (action != 0)
-	pop_sem_stack ();
+    if (node->children == NULL)
       return 1;
+    break;
     case htmllink:
       if (ud->format_for != browser)
 	break;
@@ -234,44 +242,42 @@ transcribe_paragraph (xmlNode * node, int action)
       return 1;
     case boxline:
       do_boxline (node);
-      if (action != 0)
-	pop_sem_stack ();
+    if (node->children == NULL)
       return 1;
+    break;
     case pagebreak:
       do_pagebreak (node);
-      if (action != 0)
-	pop_sem_stack ();
       return 1;
     case attrtotext:
       do_attrtotext (node);
-      if (action != 0)
-	pop_sem_stack ();
+    if (node->children == NULL)
       return 1;
+    break;
     case blankline:
       do_blankline ();
-      if (action != 0)
-	pop_sem_stack ();
+    if (node->children == NULL)
       return 1;
+    break;
     case linespacing:
       do_linespacing (node);
-      if (action != 0)
-	pop_sem_stack ();
+    if (node->children == NULL)
       return 1;
+    break;
     case softreturn:
       do_softreturn ();
-      if (action != 0)
-	pop_sem_stack ();
+    if (node->children == NULL)
       return 1;
+    break;
     case newpage:
       do_newpage ();
-      if (action != 0)
-	pop_sem_stack ();
+    if (node->children == NULL)
       return 1;
+    break;
     case righthandpage:
       do_righthandpage ();
-      if (action != 0)
-	pop_sem_stack ();
+    if (node->children == NULL)
       return 1;
+    break;
     case code:
       transcribe_computerCode (node, 0);
       if (action != 0)
@@ -287,6 +293,12 @@ transcribe_paragraph (xmlNode * node, int action)
       if (action != 0)
 	pop_sem_stack ();
       return 1;
+    case utddispimg:
+    case utdinlnimg:
+      do_utdxxxximg (node);
+      if (action != 0)
+	pop_sem_stack ();
+      return 1;
     case chemistry:
       transcribe_chemistry (node, 0);
       if (action != 0)
@@ -299,7 +311,9 @@ transcribe_paragraph (xmlNode * node, int action)
       return 1;
     case changetable:
       change_table (node);
+    if (node->children == NULL)
       return 1;
+    break;
     case pagenum:
       do_pagenum ();
       break;
