@@ -26,24 +26,25 @@
    the Free Software Foundation, 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
 
-   Maintained by John J. Boyer john.boyer@jjb-software.com
+   Maintained by John J. Boyer john.boyer@abilitiessoft.com
    */
 
 #include <stdio.h>
 #include <string.h>
 #include "louisutdml.h"
 
-static int findBrlNodes (xmlNode * node);
-static int brfDoBrlNode (xmlNode * node, int action);
-static int beginDocument ();
-static int finishBrlNode ();
-static int finishDocument ();
-static int doUtdbrlonly (xmlNode * node, int action);
-static int doUtdnewpage (xmlNode * node);
-static int doUtdnewline (xmlNode * node);
-static int doUtddispimg (xmlNode * node);
+static int brf_beginDocument ();
+static int brf_findBrlNodes (xmlNode * node);
+static int brf_doBrlNode (xmlNode * node, int action);
+static int brf_saveBuffer ();
+static int brf_doUtdbrlonly (xmlNode * node, int action);
+static int brf_doUtdnewpage (xmlNode * node);
+static int brf_doUtdnewline (xmlNode * node);
+static int brf_finishDocument ();
+
 static int firstPage;
 static int firstLineOnPage;
+static int prevLinePos;
 
 int
 utd2brf (xmlNode * node)
@@ -52,25 +53,26 @@ utd2brf (xmlNode * node)
   ud->style_top = -1;
   firstPage = 1;
   firstLineOnPage = 1;
-  beginDocument ();
-  findBrlNodes (node);
-  finishDocument ();
+  brf_beginDocument ();
+  brf_findBrlNodes (node);
+  brf_finishDocument ();
   return 1;
 }
 
 static int
-beginDocument ()
+brf_beginDocument ()
 {
+  return 1;
 }
 
 static int
-finishDocument ()
+brf_finishDocument ()
 {
+  return 1;
 }
 
-
 static int
-findBrlNodes (xmlNode * node)
+brf_findBrlNodes (xmlNode * node)
 {
   xmlNode *child;
   if (node == NULL)
@@ -81,7 +83,7 @@ findBrlNodes (xmlNode * node)
     case utdmeta:
       return 1;
     case utdbrl:
-      brfDoBrlNode (node, 0);
+      brf_doBrlNode (node, 0);
       pop_sem_stack ();
       return 1;
     default:
@@ -93,7 +95,7 @@ findBrlNodes (xmlNode * node)
       switch (child->type)
 	{
 	case XML_ELEMENT_NODE:
-	  findBrlNodes (child);
+	  brf_findBrlNodes (child);
 	  break;
 	case XML_TEXT_NODE:
 	  break;
@@ -109,19 +111,24 @@ findBrlNodes (xmlNode * node)
 static char *blanks =
   "                                                            ";
 static int
-insertCharacters (const char *text, int length)
+brf_insertCharacters (const char *text, int length)
 {
   int k;
+  if (length <= 0 || (ud->outbuf1_len_so_far + length) > 
+  ud->outbuf1_len)
+    return 0;
   for (k = 0; k < length; k++)
     ud->outbuf1[ud->outbuf1_len_so_far++] = text[k];
   return 1;
 }
 
 static int
-doDotsText (xmlNode * node)
+brf_doDotsText (xmlNode * node)
 {
   ud->text_length = 0;
   insert_utf8 (node->content);
+  if ((ud->outbuf1_len_so_far + ud->text_length) > ud->outbuf1_len)
+    brf_saveBuffer ();
   if (!lou_dotsToChar (ud->main_braille_table, ud->text_buffer,
 		       &ud->outbuf1[ud->outbuf1_len_so_far],
 		       ud->text_length, 0))
@@ -131,24 +138,22 @@ doDotsText (xmlNode * node)
 }
 
 static int
-doUtdbrlonly (xmlNode * node, int action)
+brf_doUtdbrlonly (xmlNode * node, int action)
 {
   xmlNode *child;
   if (node == NULL)
     return 0;
-  if (ud->top == 0)
-    action = 1;
   if (action != 0)
     push_sem_stack (node);
   switch (ud->stack[ud->top])
     {
     case utdnewpage:
-      doUtdnewpage (node);
+      brf_doUtdnewpage (node);
       if (action != 0)
 	pop_sem_stack ();
       return 1;
     case utdnewline:
-      doUtdnewline (node);
+      brf_doUtdnewline (node);
       if (action != 0)
 	pop_sem_stack ();
       return 1;
@@ -159,7 +164,7 @@ doUtdbrlonly (xmlNode * node, int action)
       return 1;
     case changetable:
       change_table (node);
-      return 1;
+      break;
     default:
       break;
     }
@@ -169,10 +174,13 @@ doUtdbrlonly (xmlNode * node, int action)
       switch (child->type)
 	{
 	case XML_ELEMENT_NODE:
-	  doUtdbrlonly (child, 1);
+	  brf_doUtdbrlonly (child, 1);
 	  break;
 	case XML_TEXT_NODE:
-	  doDotsText (child);
+	  if (ud->stack[ud->top] == utdbrlonly)
+	    /* print text */
+	    break;
+	  brf_doDotsText (child);
 	  break;
 	default:
 	  break;
@@ -184,44 +192,60 @@ doUtdbrlonly (xmlNode * node, int action)
   return 1;
 }
 
-static int lastLinepos;
-
 static int
-doUtdnewpage (xmlNode * node)
+brf_doUtdnewpage (xmlNode * node)
 {
-  lastLinepos = ud->page_top;
   firstLineOnPage = 1;
   if (firstPage)
     {
       firstPage = 0;
       return 1;
     }
-  insertCharacters (ud->lineEnd, strlen (ud->lineEnd));
-  insertCharacters (ud->pageEnd, strlen (ud->pageEnd));
+    for (; ud->outbuf1_len_so_far > 0 
+    && ud->outbuf1[ud->outbuf1_len_so_far - 1] <= ' ';
+	 ud->outbuf1_len_so_far--);
+  brf_insertCharacters (ud->lineEnd, strlen (ud->lineEnd));
+  brf_insertCharacters (ud->pageEnd, strlen (ud->pageEnd));
+  brf_saveBuffer ();
   return 1;
 }
 
 static int
-doUtdnewline (xmlNode * node)
+brf_doUtdnewline (xmlNode * node)
 {
   char *xy;
   int k;
   int leadingBlanks;
-  int linepos;
-  if (!firstLineOnPage)
-    insertCharacters (ud->lineEnd, strlen (ud->lineEnd));
-  xy = (char *) xmlGetProp (node, (xmlChar *) "xy");
-  for (k = 0; xy[k] != ','; k++);
-  leadingBlanks = (atoi (xy) - ud->left_margin) / ud->cell_width;
-  linepos = (atoi (&xy[k + 1]) - ud->page_top) / ud->normal_line;
-  insertCharacters (blanks, leadingBlanks);
+  int linePos;
+    for (; ud->outbuf1_len_so_far > 0 
+    && ud->outbuf1[ud->outbuf1_len_so_far - 1] <= ' ';
+	 ud->outbuf1_len_so_far--);
   if (firstLineOnPage)
-    firstLineOnPage = 0;
+    {
+      prevLinePos = ud->page_top;
+      firstLineOnPage = 0;
+    }
+  xy = (char *) xmlGetProp (node, (xmlChar *) "xy");
+  k = atoi (xy);
+  leadingBlanks = (k - ud->left_margin) / ud->cell_width;
+  if (leadingBlanks < 0 || leadingBlanks > ud->cells_per_line)
+    leadingBlanks = 0;
+  for (k = 0; xy[k] != ','; k++);
+  linePos = atoi (&xy[k + 1]);
+  if (linePos < ud->page_top)
+    linePos = ud->page_top;
+  if (linePos > ud->page_bottom)
+    linePos = ud->page_bottom;
+  /* The following will need modification for wide lines and graphics. */
+  for (k = prevLinePos; k < linePos; k += ud->normal_line)
+    brf_insertCharacters (ud->lineEnd, strlen (ud->lineEnd));
+  prevLinePos = linePos;
+  brf_insertCharacters (blanks, leadingBlanks);
   return 1;
 }
 
 int
-brfDoBrlNode (xmlNode * node, int action)
+brf_doBrlNode (xmlNode * node, int action)
 {
   xmlNode *child;
   if (node == NULL)
@@ -235,20 +259,21 @@ brfDoBrlNode (xmlNode * node, int action)
     case markhead:
       if (ud->head_node == NULL)
 	ud->head_node = node;
-      pop_sem_stack ();
-      break;
+      if (action != 0)
+        pop_sem_stack ();
+      return 1;
     case utdbrlonly:
-      doUtdbrlonly (node, 0);
+      brf_doUtdbrlonly (node, 0);
       if (action != 0)
 	pop_sem_stack ();
       return 1;
     case utdnewpage:
-      doUtdnewpage (node);
+      brf_doUtdnewpage (node);
       if (action != 0)
 	pop_sem_stack ();
       return 1;
     case utdnewline:
-      doUtdnewline (node);
+      brf_doUtdnewline (node);
       if (action != 0)
 	pop_sem_stack ();
       return 1;
@@ -259,7 +284,7 @@ brfDoBrlNode (xmlNode * node, int action)
       return 1;
     case changetable:
       change_table (node);
-      return 1;
+      break;
     default:
       break;
     }
@@ -269,10 +294,10 @@ brfDoBrlNode (xmlNode * node, int action)
       switch (child->type)
 	{
 	case XML_ELEMENT_NODE:
-	  brfDoBrlNode (child, 1);
+	  brf_doBrlNode (child, 1);
 	  break;
 	case XML_TEXT_NODE:
-	  doDotsText (child);
+	  brf_doDotsText (child);
 	  break;
 	default:
 	  break;
@@ -284,15 +309,16 @@ brfDoBrlNode (xmlNode * node, int action)
       pop_sem_stack ();
       return 1;
     }
-  finishBrlNode ();
+  brf_saveBuffer ();
   return 1;
 }
 
 static int
-finishBrlNode ()
+brf_saveBuffer ()
 {
-  if (ud->outbuf1_len_so_far == 0)
+  if (ud->outbuf1_len_so_far <= 0)
     return 1;
   write_buffer (1, 0);
+  ud->outbuf1_len_so_far = 0;
   return 1;
 }
